#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
from __future__ import print_function

import os
import re
import sys
import subprocess
import glob
import SPARQLWrapper
import sets

DBpediaFormat="<http://dbpedia.org/resource/%s>" 
#DBpediaFormat="dbr:%s" 

def toDBpediaResource(label):
    return DBpediaFormat % (label.replace(" ", "_").capitalize());

def toUpperDBpediaResource(label):
    return DBpediaFormat % (label.replace(" ", "_").upper());


#check if query result matches answer

if(len(sys.argv) != 6):
    exit("python2 searchDBpedia.py <sparqlAddress> <labelsFilename> <propositionsFilename> <failedFilename> <option>")

        
sparqlAddress = sys.argv[1]
labelsFilename = sys.argv[2]
propositionsFilename = sys.argv[3]
failedFilename = sys.argv[4]
option = sys.argv[5]

eliminateEmDash="eliminateEmDash"
emDash="—"
resourceMatch="resourceMatch"
allUpperResourceMatch="allUpperResourceMatch"

#prepare connection to store
sparql = SPARQLWrapper.SPARQLWrapper(sparqlAddress)

#TODO how to bind variables in query
#TODO - would be useful, because here we duplicate the same resource

#prefix dbr: <http://dbpedia.org/resource/>
#3rd tripple pattern:
#isBlank is a hack to use lod.openlinksw.com/sparql endpoint
#5th tripple pattern:
#sometimes disambiguation page
#does not have the things it disambiguates listed,
#but it still has: 
# is dbo:wikiPageRedirects of | dbr:..._(disambiguation)
mainResourceMatchQuery = """
prefix dbo: <http://dbpedia.org/ontology/>
ask
{
	%s ?p ?o
	. filter (! exists { %s dbo:wikiPageRedirects ?w })
	. filter (! isBlank(?o))
        . filter (! exists { %s dbo:wikiPageDisambiguates ?u})
        . filter (! exists { ?z dbo:wikiPageRedirects %s . filter( strends( str(?z), '_(disambiguation)')) } )
}
"""


#labels from file are already in lower-case
labelMatchQuery = """
prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>
select ?uri
{
   ?uri rdfs:label ?label .
   filter( lcase(?label) = "%s")
}
"""


with open(labelsFilename) as labelsFile, open(propositionsFilename, 'w') as propositionsFile, open(failedFilename, 'w') as failedFile:
    for label in [line.rstrip('\n') for line in labelsFile.readlines()]:

        if(option==eliminateEmDash):
            if(emDash in label):
                print(label, file=failedFile) 
            else:
                print(label, file=propositionsFile) 

        elif(option==resourceMatch):
            resource = toDBpediaResource(label);
            query = mainResourceMatchQuery % (resource, resource, resource, resource)
            sparql.setQuery(query)
            sparql.setReturnFormat(SPARQLWrapper.JSON)
            results = sparql.query().convert()
            decision = results["boolean"]
            if(decision):
                print("%s | %s" % (label, resource), file=propositionsFile) 
                #we flush, because we want to read results while program is working
                propositionsFile.flush()
            else:
                print(label, file=failedFile) 
                failedFile.flush()
        #TODO refactor
        elif(option==allUpperResourceMatch):
            resource = toUpperDBpediaResource(label);
            query = mainResourceMatchQuery % (resource, resource, resource, resource)
            sparql.setQuery(query)
            sparql.setReturnFormat(SPARQLWrapper.JSON)
            results = sparql.query().convert()
            decision = results["boolean"]
            if(decision):
                print("%s | %s" % (label, resource), file=propositionsFile) 
                #we flush, because we want to read results while program is working
                propositionsFile.flush()
            else:
                print(label, file=failedFile) 
                failedFile.flush()
            



#    query = [line.rstrip('\n') for line in queryFile.readlines() if not line.startswith('#')]
#    query.insert(0, "DEFINE input:inference 'lubmruleset'")
#    queryString = "\n".join(query)
#    
#    sparql.setQuery(queryString)
#    sparql.setReturnFormat(SPARQLWrapper.JSON)
#    results = sparql.query().convert()
#    for result in results["results"]["bindings"]:
#        sparqlOutput.append(" ".join([k + ":" + result[k]["value"] + " " for k in sorted(result.keys())]))
#
#
#
#    #parse answers into rows + variables names
#    answerFileName = os.path.join(answerDirectory, ANSWER_FILE_PREFIX + os.path.split(queryFileName)[1])
#    expectedOutput = None
#    with open(answerFileName) as answerFile:
#        answerLines = answerFile.readlines();
#        #check if answer should be empty set
#        if answerLines[0].strip() == NO_ANSWERS:
#            if len(sparqlOutput) == 0:
#                print("OK")
#            else:
#                print("ERR: received result where expected no result")
#            continue
#        
#        variableNames = answerLines.pop(0).split()
#
#        expectedOutput = []
#        for line in answerLines:
#            values = line.split()
#
#            valDict = {} 
#            for i in range(len(values)):
#                valDict[variableNames[i]] = values[i]
#            #TODO refactor to function that compares two lists of dictionaries
#            #however, there is probem because the first one is a dict of dicts...
#            expectedOutput.append(" ".join([k + ":" + valDict[k] + " " for k in sorted(valDict.keys())]))
#
#    #compare sizes
#    sizeDifference = len(expectedOutput) - len(sparqlOutput)
#    if sizeDifference != 0:
#        print("ERR: results differ in length: expected: %s, sparql: %s" % (len(expectedOutput), len(sparqlOutput)))
#
#    #there is a special case when
#    expectedOutputSet = set(sorted(expectedOutput))
#    sparqlOutputSet = set(sorted(sparqlOutput))
#    if(expectedOutputSet == sparqlOutputSet):
#        if(sizeDifference != 0):
#            print("ERR: results differ because of different number of duplicated triples.")
#        else:
#            print("OK")
#    else:
#        print("ERR: content of results differs.")
#        onlyInSparql = sparqlOutputSet - expectedOutputSet
#        onlyInExpected = expectedOutputSet - sparqlOutputSet
#        for element in onlyInSparql:
#            print("ERR: only in sparql: %s" % element)
#        for element in onlyInExpected:
#            print("ERR: only in expected: %s" % element)
#
#
